@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -561,6 +561,18 @@ sub IsMessageContentType {
     return $self->ContentType =~ m{^\s*message/}i ? 1 : 0;
 }
 
+=head2 IsAttachmentContentDisposition
+
+Returns a boolean indicating if the Content-Disposition of this attachment
+is a MIME attachment.
+
+=cut
+
+sub IsAttachmentContentDisposition {
+    my $self = shift;
+    return ( $self->GetHeader('Content-Disposition') // '' ) =~ m{^\s*attachment\b}i ? 1 : 0;
+}
+
 =head2 Addresses
 
 Returns a hashref of all addresses related to this attachment.
@@ -891,6 +903,47 @@ sub _SplitHeaders {
     return(@headers);
 }
 
+=head2 CryptStatus
+
+Returns the parsed status from the X-RT-GnuPG-Status or
+X-RT-SMIME-Status header.
+
+The return value is an array of hashrefs; each hashref is as described
+in L<RT::Crypt::ParseStatus>; however, each hashref has one additional
+entry 'Protocol' which is the name of the crypto protocol used and is
+one of 'SMIME' or 'GnuPG'.
+
+If no crypto header exists, returns an empty array
+
+=cut
+
+sub CryptStatus {
+    my $self = shift;
+    my @ret  = ();
+
+    foreach my $h ( $self->SplitHeaders ) {
+        next unless $h =~ /^X-RT-(GnuPG|SMIME)-Status:/i;
+        my $protocol = $1;
+        my ( $h_key, $h_val ) = split( /:\s*/, $h, 2 );
+        my @result = RT::Crypt->ParseStatus(
+            Protocol => $protocol,
+            Status   => $h_val
+        );
+
+        # Canonicalize protocol case so it's always SMIME or GnuPG
+        if ( uc($protocol) eq 'SMIME' ) {
+            $protocol = 'SMIME';
+        }
+        elsif ( uc($protocol) eq 'GNUPG' ) {
+            $protocol = 'GnuPG';
+        }
+        foreach my $hash (@result) {
+            $hash->{'Protocol'} = $protocol;
+            push( @ret, $hash );
+        }
+    }
+    return @ret;
+}
 
 sub Encrypt {
     my $self = shift;
@@ -919,9 +972,9 @@ sub Encrypt {
         RT->Config->Get('CorrespondAddress'),
         RT->Config->Get('CommentAddress'),
     ) {
-        my %res = RT::Crypt->GetKeysInfo( Key => $address, Type => 'private' );
+        my %res = RT::Crypt->GetKeysInfo( Queue => $queue, Key => $address, Type => 'private' );
         next if $res{'exit_code'} || !$res{'info'};
-        %res = RT::Crypt->GetKeysForEncryption( $address );
+        %res = RT::Crypt->GetKeysForEncryption( Queue => $queue, Recipient => $address );
         next if $res{'exit_code'} || !$res{'info'};
         $encrypt_for = $address;
     }
@@ -931,6 +984,7 @@ sub Encrypt {
 
     my $content = $self->Content;
     my %res = RT::Crypt->SignEncryptContent(
+        Queue => $queue,
         Content => \$content,
         Sign => 0,
         Encrypt => 1,
