@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -94,6 +94,18 @@ Set to true value to enable this subsystem:
         ... other options ...
     );
 
+=head3 Setting the GnuPG Command
+
+By default, RT looks for a command named C<gpg> in your C<$PATH> to run GnuPG.
+If the command has a different name or is outside the C<$PATH> on your system,
+you can specify the full path of the GnuPG command using the C<GnuPG> option:
+
+    Set( %GnuPG,
+        Enable => 1,
+        GnuPG  => '/usr/local/bin/gpg2',
+        ... other options ...
+    );
+
 =head3 Format of outgoing messages
 
 The format of outgoing messages can be controlled using the
@@ -174,7 +186,7 @@ Common options include:
 =item --homedir
 
 The GnuPG home directory where the keyrings are stored; by default it is
-set to F</opt/rt4/var/data/gpg>.
+set to F</opt/rt5/var/data/gpg>.
 
 You can manage this data with the 'gpg' commandline utility using the
 GNUPGHOME environment variable or C<--homedir> option.  Other utilities may
@@ -287,8 +299,6 @@ my %supported_opt = map { $_ => 1 } qw(
        verbose
 );
 
-our $RE_FILE_EXTENSIONS = qr/pgp|asc/i;
-
 # DEV WARNING: always pass all STD* handles to GnuPG interface even if we don't
 # need them, just pass 'IO::Handle->new()' and then close it after safe_run_child.
 # we don't want to leak anything into FCGI/Apache/MP handles, this break things.
@@ -825,12 +835,16 @@ sub FindScatteredParts {
         }
     }
 
+    my $file_extension_regex = join '|', @{ RT->Config->Get('GnuPG')->{FileExtensions} };
     # attachments with inline encryption
     foreach my $part ( @parts ) {
         next if $args{'Skip'}{$part};
 
         my $fname = $part->head->recommended_filename || '';
-        next unless $fname =~ /\.${RE_FILE_EXTENSIONS}$/;
+        next unless $fname =~ /\.(?:$file_extension_regex)$/;
+
+        # skip pgp keys, which could have .asc file extension
+        next if ( $part->head->mime_type // '' ) eq 'application/pgp-keys';
 
         $RT::Logger->debug("Found encrypted attachment '$fname'");
 
@@ -849,7 +863,7 @@ sub FindScatteredParts {
         my $type = $self->_CheckIfProtectedInline( $part );
         next unless $type;
 
-        my $file = ($part->head->recommended_filename||'') =~ /\.${RE_FILE_EXTENSIONS}$/;
+        my $file = ($part->head->recommended_filename||'') =~ /\.(?:$file_extension_regex)$/;
 
         $args{'Skip'}{$part} = 1;
         push @res, {
@@ -993,7 +1007,7 @@ sub VerifyRFC3156 {
     my %args = ( Data => undef, Signature => undef, @_ );
 
     my ($tmp_fh, $tmp_fn) = File::Temp::tempfile( UNLINK => 1 );
-    binmode $tmp_fh, ':raw:eol(CRLF?)';
+    binmode $tmp_fh, ':raw:eol(CRLF)';
     $args{'Data'}->print( $tmp_fh );
     $tmp_fh->flush;
 
@@ -1044,7 +1058,7 @@ sub DecryptRFC3156 {
 
     seek $tmp_fh, 0, 0;
     my $parser = RT::EmailParser->new();
-    my $decrypted = $parser->ParseMIMEEntityFromFileHandle( $tmp_fh, 0 );
+    my $decrypted = $parser->ParseMIMEEntityFromFileHandle( $tmp_fh, 0, 1 );
     $decrypted->{'__store_link_to_object_to_avoid_early_cleanup'} = $parser;
 
     $args{'Top'}->parts( [$decrypted] );
@@ -1236,7 +1250,8 @@ sub DecryptAttachment {
     }
 
     my $filename = $embedded_fn || $head->recommended_filename;
-    $filename =~ s/\.${RE_FILE_EXTENSIONS}$//i;
+    my $file_extension_regex = join '|', @{ RT->Config->Get('GnuPG')->{FileExtensions} };
+    $filename =~ s/\.(?:$file_extension_regex)$//i;
     $head->mime_attr( $_ => $filename )
         foreach (qw(Content-Type.name Content-Disposition.filename));
 
