@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -172,9 +172,9 @@ sub _RegisterAsRole {
             $role->Load($id);
 
             if ($object->isa('RT::Queue')) {
-                # there's no way to apply the custom
-                # role to a queue before that queue is created
-                return 0;
+                # In case queue level custom role groups got deleted
+                # somehow.  Allow to re-create them like default ones.
+                return $role->IsAdded($object->id);
             }
             elsif ($object->isa('RT::Ticket')) {
                 # see if the role has been applied to the ticket's queue
@@ -191,21 +191,19 @@ sub _RegisterAsRole {
         AppliesToObjectPredicate => sub {
             my $object = shift;
 
-            # reload the role to avoid capturing $self across requests
-            my $role = RT::CustomRole->new(RT->SystemUser);
-            $role->Load($id);
-
-            return 0 if $role->Disabled;
+            # for callers not specific to any queue, e.g. ColumnMap
+            if (!ref($object)) {
+                return 1;
+            }
 
             # all roles are also available on RT::System for granting rights
             if ($object->isa('RT::System')) {
                 return 1;
             }
 
-            # for callers not specific to any queue, e.g. ColumnMap
-            if (!ref($object)) {
-                return 1;
-            }
+            # reload the role to avoid capturing $self across requests
+            my $role = RT::CustomRole->new(RT->SystemUser);
+            $role->Load($id);
 
             if ( $object->isa('RT::Ticket') || $object->isa('RT::Queue') ) {
                 return 0 unless $object->CurrentUserHasRight('SeeQueue');
@@ -413,7 +411,8 @@ sub AddToObject {
         unless $queue->CurrentUserHasRight('AdminCustomRoles');
     my $rec = RT::ObjectCustomRole->new( $self->CurrentUser );
     my ( $status, $add ) = $rec->Add( %args, CustomRole => $self );
-    my $msg = $self->loc("[_1] added to queue [_2]", $self->Name, $queue->Name) if $status;
+    my $msg;
+    $msg = $self->loc("[_1] added to queue [_2]", $self->Name, $queue->Name) if $status;
 
     return ( $add, $msg );
 }
@@ -452,7 +451,8 @@ sub RemoveFromObject {
     $rec->LoadByCols( CustomRole => $self->id, ObjectId => $args{'ObjectId'} );
     return (0, $self->loc('Custom role is not added') ) unless $rec->id;
     my ( $status, $delete ) = $rec->Delete;
-    my $msg = $self->loc("[_1] removed from queue [_2]", $self->Name, $queue->Name) if $status;
+    my $msg;
+    $msg = $self->loc("[_1] removed from queue [_2]", $self->Name, $queue->Name) if $status;
 
     return ( $delete, $msg );
 }
@@ -679,29 +679,21 @@ sub SetDisabled {
     my $self = shift;
     my $value = shift;
 
-    $RT::Handle->BeginTransaction();
-
     my ($ok, $msg) = $self->_Set( Field => 'Disabled', Value => $value );
     unless ($ok) {
-        $RT::Handle->Rollback();
         $RT::Logger->warning("Couldn't ".(($value == 0) ? "enable" : "disable")." custom role ".$self->Name.": $msg");
         return ($ok, $msg);
     }
 
-    # we can't unconditionally re-enable all role groups because
-    # if you add a role to queues A and B, add users and privileges and
-    # tickets on both, remove the role from B, disable the role, then re-enable
-    # the role, we shouldn't re-enable B because it's still removed
-    my $queues = $self->AddedTo;
-    while (my $queue = $queues->Next) {
-        $self->_SetGroupsDisabledForQueue($value, $queue);
-    }
-
-    $RT::Handle->Commit();
+    RT::Principal->InvalidateACLCache();
 
     if ( $value == 0 ) {
+        $self->_RegisterAsRole;
+        RT->System->CustomRoleCacheNeedsUpdate(1);
         return (1, $self->loc("Custom role enabled"));
     } else {
+        $self->_UnregisterAsRole;
+        RT->System->CustomRoleCacheNeedsUpdate(1);
         return (1, $self->loc("Custom role disabled"));
     }
 }
