@@ -3,7 +3,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -74,12 +74,11 @@ use warnings;
 use base 'RT::Record';
 
 use Role::Basic 'with';
-with "RT::Record::Role::Rights";
+with "RT::Record::Role::Rights",
+     "RT::Record::Role::Links";
 
 sub Table {'Groups'}
 
-
-
 use RT::Users;
 use RT::GroupMembers;
 use RT::Principals;
@@ -96,6 +95,7 @@ __PACKAGE__->AddRight( Staff => SeeGroupDashboard    => 'View group dashboards')
 __PACKAGE__->AddRight( Admin => CreateGroupDashboard => 'Create group dashboards'); # loc
 __PACKAGE__->AddRight( Admin => ModifyGroupDashboard => 'Modify group dashboards'); # loc
 __PACKAGE__->AddRight( Admin => DeleteGroupDashboard => 'Delete group dashboards'); # loc
+__PACKAGE__->AddRight( Staff => ModifyGroupLinks     => 'Modify group links' ); # loc
 
 =head1 METHODS
 
@@ -306,10 +306,20 @@ sub _Create {
         @_
     );
 
-    # Enforce uniqueness on user defined group names
-    if ($args{'Domain'} and $args{'Domain'} eq 'UserDefined') {
-        my ($ok, $msg) = $self->_ValidateUserDefinedName($args{'Name'});
-        return ($ok, $msg) if not $ok;
+    $args{'Name'} = $self->CanonicalizeName( $args{'Name'} );
+
+    if ($args{'Domain'}) {
+        # Enforce uniqueness on user defined group names
+        if ($args{'Domain'} eq 'UserDefined') {
+            my ($ok, $msg) = $self->_ValidateUserDefinedName($args{'Name'});
+            return ($ok, $msg) if not $ok;
+        }
+
+        # Enforce uniqueness on SystemInternal and system role groups
+        if ($args{'Domain'} eq 'SystemInternal' || $args{'Domain'} eq 'RT::System-Role') {
+            my ($ok, $msg) = $self->_ValidateNameForDomain($args{'Name'}, $args{'Domain'});
+            return ($ok, $msg) if not $ok;
+        }
     }
 
     $RT::Handle->BeginTransaction() unless ($args{'InsideTransaction'});
@@ -400,6 +410,30 @@ sub ValidateName {
     return $self->SUPER::ValidateName($value);
 }
 
+=head2 _ValidateNameForDomain VALUE DOMAIN
+
+Returns true if the group name isn't in use in the same domain, false otherwise.
+
+=cut
+
+sub _ValidateNameForDomain {
+    my ($self, $value, $domain) = @_;
+
+    return (0, 'Name is required') unless length $value;
+
+    my $dupcheck = RT::Group->new(RT->SystemUser);
+    if ($domain eq 'UserDefined') {
+        $dupcheck->LoadUserDefinedGroup($value);
+    }
+    else {
+        $dupcheck->LoadByCols(Domain => $domain, Name => $value);
+    }
+    if ( $dupcheck->id && ( !$self->id || $self->id != $dupcheck->id ) ) {
+        return ( 0, $self->loc( "Group name '[_1]' is already in use", $value ) );
+    }
+    return 1;
+}
+
 =head2 _ValidateUserDefinedName VALUE
 
 Returns true if the user defined group name isn't in use, false otherwise.
@@ -409,14 +443,7 @@ Returns true if the user defined group name isn't in use, false otherwise.
 sub _ValidateUserDefinedName {
     my ($self, $value) = @_;
 
-    return (0, 'Name is required') unless length $value;
-
-    my $dupcheck = RT::Group->new(RT->SystemUser);
-    $dupcheck->LoadUserDefinedGroup($value);
-    if ( $dupcheck->id && ( !$self->id || $self->id != $dupcheck->id ) ) {
-        return ( 0, $self->loc( "Group name '[_1]' is already in use", $value ) );
-    }
-    return 1;
+    return $self->_ValidateNameForDomain($value, 'UserDefined');
 }
 
 =head2 _CreateACLEquivalenceGroup { Principal }
@@ -624,7 +651,7 @@ sub SetName {
     my $self = shift;
     my $value = shift;
 
-    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $value );
+    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $self->CanonicalizeName($value) );
     return ($status, $msg);
 }
 
@@ -1751,6 +1778,36 @@ sub _CustomRoleObj {
     return;
 }
 
+sub ModifyLinkRight {'ModifyGroupLinks'}
+
+=head2 URI
+
+Returns this group's URI
+
+=cut
+
+sub URI {
+    my $self = shift;
+
+    require RT::URI::group;
+    my $uri = RT::URI::group->new($self->CurrentUser);
+    return $uri->URIForObject($self);
+}
+
+=head2 CanonicalizeName NAME
+
+Strip leading/trailing spaces and returns the updated name.
+
+=cut
+
+sub CanonicalizeName {
+    my $self = shift;
+    my $name = shift // return undef;
+    $name =~ s!^\s+!!;
+    $name =~ s!\s+$!!;
+    return $name;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
