@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -371,6 +371,7 @@ sub LoadByCols {
 
     # We don't want to hang onto this
     $self->ClearAttributes;
+    delete $self->{_Roles};
 
     unless ( $self->_Handle->CaseSensitive ) {
         my ( $ret, $msg ) = $self->SUPER::LoadByCols( @_ );
@@ -2033,7 +2034,7 @@ sub _AddCustomFieldValue {
         );
 
         unless ( $new_value_id ) {
-            return ( 0, $self->loc( "Could not add new custom field value: [_1]", $value_msg ) );
+            return ( 0, $self->loc( "Could not add a new value to custom field '[_1]': [_2]", $cf->Name, $value_msg ) );
         }
 
         my $new_value = RT::ObjectCustomFieldValue->new( $self->CurrentUser );
@@ -2050,7 +2051,8 @@ sub _AddCustomFieldValue {
               );
         }
 
-        my $new_content = $new_value->Content;
+        # Fall back to '' in case current user doesn't have rights.
+        my $new_content = $new_value->Content // '';
 
         # For datetime, we need to display them in "human" format in result message
         #XXX TODO how about date without time?
@@ -2126,19 +2128,23 @@ Add default values to object's empty custom fields.
 
 sub AddCustomFieldDefaultValues {
     my $self = shift;
-    my $cfs  = $self->CustomFields;
+
+    my $object = ( ref $self )->new( RT->SystemUser );
+    $object->Load( $self->id );
+
+    my $cfs  = $object->CustomFields;
     my @msgs;
     while ( my $cf = $cfs->Next ) {
-        next if $self->CustomFieldValues($cf->id)->Count || !$cf->SupportDefaultValues;
+        next if $object->CustomFieldValues($cf->id)->Count || !$cf->SupportDefaultValues;
         my ( $on ) = grep { $_->isa( $cf->RecordClassFromLookupType ) } $cf->ACLEquivalenceObjects;
         my $values = $cf->DefaultValues( Object => $on || RT->System );
         foreach my $value ( UNIVERSAL::isa( $values => 'ARRAY' ) ? @$values : $values ) {
-            next if $self->CustomFieldValueIsEmpty(
+            next if $object->CustomFieldValueIsEmpty(
                 Field => $cf,
                 Value => $value,
             );
 
-            my ( $status, $msg ) = $self->_AddCustomFieldValue(
+            my ( $status, $msg ) = $object->_AddCustomFieldValue(
                 Field             => $cf->id,
                 Value             => $value,
                 RecordTransaction => 0,
@@ -2742,9 +2748,14 @@ sub FindDependencies {
         }
     }
 
+    my $objs;
+
     # Object attributes, we have to check on every object
-    my $objs = $self->Attributes;
-    $deps->Add( in => $objs );
+    # attributes of attributes are not supported yet though.
+    if ( !$self->isa('RT::Attribute') ) {
+        $objs = $self->Attributes;
+        $deps->Add( in => $objs );
+    }
 
     # Transactions
     if (   $self->isa("RT::Ticket")
@@ -2753,6 +2764,7 @@ sub FindDependencies {
         or $self->isa("RT::Article")
         or $self->isa("RT::Asset")
         or $self->isa("RT::Catalog")
+        or $self->isa("RT::Attribute")
         or $self->isa("RT::Queue") )
     {
         $objs = RT::Transactions->new( $self->CurrentUser );
@@ -2852,6 +2864,9 @@ sub PreInflate {
     my $class = shift;
     my ($importer, $uid, $data) = @_;
 
+    # In case it's RT::Class from RT 4 that has HotList column
+    delete $data->{HotList} if $uid =~ /^RT::Class-/;
+
     my $ca = $class->_ClassAccessible;
     my %ca = %{ $ca };
 
@@ -2922,7 +2937,7 @@ sub _AsInsertQuery
 
     my $dbh = $RT::Handle->dbh;
 
-    my $res = "INSERT INTO ". $dbh->quote_identifier( $self->Table );
+    my $res = "INSERT INTO ". $self->Table;
     my $values = $self->{'values'};
     $res .= "(". join( ",", map { $dbh->quote_identifier( $_ ) } sort keys %$values ) .")";
     $res .= " VALUES";
