@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -83,35 +83,29 @@ requires 'LifecycleType';
 
 =head1 PROVIDES
 
-=head2 LifecycleObj [CONTEXT_OBJ]
+=head2 LifecycleObj
 
 Returns an L<RT::Lifecycle> object for this record's C<Lifecycle>.  If called
 as a class method, returns an L<RT::Lifecycle> object which is an aggregation
-of all lifecycles of the appropriate type. Pass an additional L<CONTEXT_OBJ>
-to check rights at the objects context.
+of all lifecycles of the appropriate type.
 
 =cut
 
 sub LifecycleObj {
     my $self = shift;
-    my $context_obj = shift || undef;
     my $type = $self->LifecycleType;
-    my $fallback = $self->_Accessible( Lifecycle => "default" );
 
     unless (blessed($self) and $self->id) {
         return RT::Lifecycle->Load( Type => $type );
     }
 
-    my $name = $self->Lifecycle($context_obj);
-    if ( !$name ) {
-        RT::Logger->debug('Failing back to default lifecycle value');
-        $name = $fallback;
-    }
+    my $name = $self->__Value('Lifecycle');
     my $res  = RT::Lifecycle->Load( Name => $name, Type => $type );
     unless ( $res ) {
         RT->Logger->error(
             sprintf "Lifecycle '%s' of type %s for %s #%d doesn't exist",
                     $name, $type, ref($self), $self->id);
+        my $fallback = $self->_Accessible( Lifecycle => "default" );
         return RT::Lifecycle->Load( Name => $fallback, Type => $type );
     }
     return $res;
