@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2021 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -125,6 +125,14 @@ sub Query {
             my $value = $self->RawValue( $column );
             my $op = '=';
             if ( defined $value ) {
+                if ( $info->{INFO} eq 'Watcher' && $info->{FIELD} eq 'id' ) {
+
+                    # convert id to name
+                    my $princ = RT::Principal->new( $self->CurrentUser );
+                    $princ->Load($value);
+                    $value = $princ->Object->Name if $princ->Object;
+                }
+
                 unless ( $value =~ /^\d+$/ ) {
                     $value =~ s/(['\\])/\\$1/g;
                     $value = "'$value'";
