@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -93,6 +93,9 @@ __PACKAGE__->AddRight( Staff   => ShowGlobalTemplates => 'Show global templates'
 __PACKAGE__->AddRight( General => LoadSavedSearch     => 'Allow loading of saved searches'); # loc
 __PACKAGE__->AddRight( General => CreateSavedSearch   => 'Allow creation of saved searches'); # loc
 __PACKAGE__->AddRight( Admin   => ExecuteCode         => 'Allow writing Perl code in templates, scrips, etc'); # loc
+__PACKAGE__->AddRight( General => SeeSelfServiceGroupTicket => 'See tickets for other group members in SelfService' ); # loc
+__PACKAGE__->AddRight( Staff   => ShowSearchAdvanced    => 'Show search "Advanced" menu' ); # loc
+__PACKAGE__->AddRight( Staff   => ShowSearchBulkUpdate  => 'Show search "Bulk Update" menu' ); # loc
 
 =head2 AvailableRights
 
@@ -235,6 +238,59 @@ sub CustomRoleCacheNeedsUpdate {
     }
 }
 
+=head2 ConfigCacheNeedsUpdate ( 1 )
+
+Attribute to decide when we need to flush the database settings
+and re-register any changes.  Set when settings are created, enabled/disabled, etc.
+
+If passed a true value, will update the attribute to be the current time.
+
+=cut
+
+sub ConfigCacheNeedsUpdate {
+    my $self = shift;
+    my $time = shift;
+
+    if ($time) {
+        return $self->SetAttribute(Name => 'ConfigCacheNeedsUpdate', Content => $time);
+    } else {
+        my $cache = $self->FirstAttribute('ConfigCacheNeedsUpdate');
+        return (defined $cache ? $cache->Content : 0 );
+    }
+}
+
+# This needs to be in RT::System as RT::Interface::Web and RT::Interface::Email both use this
+my $lifecycle_cache_time = time;
+sub MaybeRebuildLifecycleCache {
+    my $needs_update = RT->System->LifecycleCacheNeedsUpdate;
+    if ( $needs_update > $lifecycle_cache_time ) {
+        RT::Lifecycle->FillCache;
+        $lifecycle_cache_time = $needs_update;
+    }
+}
+
+=head2 LifecycleCacheNeedsUpdate ( 1 )
+
+Attribute to decide when we need to flush the list of lifecycles
+and re-register any changes. This is needed for the lifecycle UI editor.
+
+If passed a true value, will update the attribute to be the current time.
+
+=cut
+
+sub LifecycleCacheNeedsUpdate {
+    my $self   = shift;
+    my $update = shift;
+
+    if ($update) {
+        return $self->SetAttribute(Name => 'LifecycleCacheNeedsUpdate', Content => time);
+    }
+    else {
+        my $cache = $self->FirstAttribute('LifecycleCacheNeedsUpdate');
+        return (defined $cache ? $cache->Content : 0);
+    }
+}
+
 =head2 AddUpgradeHistory package, data
 
 Adds an entry to the upgrade history database. The package can be either C<RT>
