@@ -2,7 +2,7 @@
 #
 # COPYRIGHT:
 #
-# This software is Copyright (c) 1996-2023 Best Practical Solutions, LLC
+# This software is Copyright (c) 1996-2022 Best Practical Solutions, LLC
 #                                          <sales@bestpractical.com>
 #
 # (Except where explicitly superseded by other copyright notices)
@@ -55,7 +55,6 @@ use base qw(RT::Test);
 use File::Temp qw(tempdir);
 use IPC::Run3 'run3';
 use File::Copy;
-use RT::Util 'safe_run_child';
 use 5.010;
 
 our @EXPORT =
@@ -146,17 +145,6 @@ sub bootstrap_more_config {
         if (!$args->{GnuPG}) {
             print $handle qq{ Set(\%GnuPG, Enable => 0); };
         }
-
-        my $out;
-        safe_run_child {
-            run3( [ $openssl, 'version' ], \undef, \$out, \undef, )
-        };
-
-        my @providers;
-        if ( $out =~ /^OpenSSL 3/ ) {
-            @providers = ( 'default', 'legacy' );
-        }
-
         print $handle qq{
         Set(\%SMIME =>
             Enable => 1,
@@ -167,7 +155,6 @@ sub bootstrap_more_config {
             OpenSSL => q{$openssl},
             Keyring => q{$keyring},
             CAPath  => q{$ca},
-            Providers => [qw(@providers)],
             );
         };
 
@@ -249,7 +236,7 @@ sub create_a_ticket {
         }
     }
 
-    $m->submit;
+    $m->click('SubmitTicket');
     is $m->status, 200, "request successful";
 
     $m->content_lacks("unable to sign outgoing email messages");
